
import mapobj, menu
from sets import Set

class Unclosed: pass

class Level:

	def __init__(s):
		s.m = mapobj.Map(' ')
		s.filename = s.levelname = None

	def set(s, x, y, thing): s.m.set(x, y, thing)
	def get(s, x, y): return s.m.get(x, y)

	def change(s, x, y, ch):
		oldch = s.get(x, y)
		if (oldch + ch) in ('.@', '@.'): ch = '+'
		elif (oldch + ch) in ('.$', '$.'): ch = '*'
		s.set(x, y, ch)
		if ch != '#':
			for xx, yy in ((x - 1, y), (x + 1, y),
					(x, y - 1), (x, y + 1)):
				s.maybe_wall(xx, yy, ch == ' ')

	def maybe_wall(s, x, y, must_be_empty):
		if not (s.get(x, y) == ' ') == must_be_empty:
			s.set(x, y, '#')

	def ask_file(s, scr):
		s.filename = menu.ask_input(scr, 'File name:')
	def maybe_ask_file(s, scr):
		if not s.filename: s.ask_file(scr)
	def ask_name(s, scr):
		s.levelname = menu.ask_input(scr, 'Level name:')
	def maybe_ask_name(s, scr):
		if not s.levelname: s.ask_name(scr)

	def mark_floors(s):
		checked = Set()
		def floor_propagate(x, y):
			cell = s.get(x, y)
			if cell in '#_': return
			if (x, y) in checked: return
			checked.add((x, y))
			if x < minx or y < miny or x >= maxx or y >= maxy:
				raise Unclosed
			if cell == ' ': s.set(x, y, '_')
			try:
				for xx, yy in ((x - 1, y), (x + 1, y),
						(x, y - 1), (x, y + 1)):
					floor_propagate(xx, yy)
			except Unclosed:
				if cell == ' ': s.set(x, y, ' ')
				raise
		minx, miny, maxx, maxy = s.m.bounds()
		for x in range(minx, maxx):
			for y in range(miny, maxy):
				if s.get(x, y) in '$@.+*':
					floor_propagate(x, y)

	def load_level(s, filename = None):
		if filename: s.filename = filename
		if not s.filename: return "No file name"
		f = open(s.filename)
		s.m = mapobj.Map(' ')
		row = col = 0
		for line in f:
			line = line.rstrip()
			if line and line[0] == "'":
				s.levelname = line[1:line.rindex("'")]
				continue
			for char in line:
				s.set(col, row, char)
				col = col + 1
			col = 0
			row = row + 1
		try: s.mark_floors()
		except Unclosed: return "Unclosed level"
		minx, miny, maxx, maxy = s.m.bounds()
		return (minx + maxx) // 2, (miny + maxy) // 2

	def save_level(s, filename = None):
		if filename: s.filename = filename
		if not s.filename: return "No file name"
		f = open(s.filename, 'w')
		def write_line(line):
			line = map(lambda c: [c, ' '][c == '_'], line)
			f.write(''.join(line) + '\n')
		f.write("'%s'\n" % s.levelname)
		s.m.row_iterate(write_line)
		return "Level '%s' saved in %s" % (s.levelname, s.filename)

